# Auto-generated integration tests (mocked) - generated by ClientTestGenerator
# Uses: pytest, respx, httpx

import pytest
import json
from pathlib import Path
import respx
from httpx import Response
import near_jsonrpc_models
from near_jsonrpc_client import NearClientSync

FIXTURES_DIR = Path(__file__).parent / "fixtures" / "json"

def load_mock_json(name: str):
    p = Path(FIXTURES_DIR) / name
    if not p.exists():
        return None
    with p.open("r", encoding="utf-8") as fh:
        return json.load(fh)

def _normalize(obj):
    """Recursively convert pydantic near_jsonrpc_models (and lists/dicts containing them)
    to JSON-serializable Python primitives comparable with fixture JSON.

    Rules:
    - If object has model_dump, call model_dump(mode='json', exclude_none=True)
    - If list, normalize each element
    - If dict, normalize values
    - Otherwise, return as-is
    """
    # pydantic model
    try:
        # check duck-typing attribute (works for pydantic v1 & v2)
        if hasattr(obj, "model_dump") or hasattr(obj, "dict"):
            if hasattr(obj, "model_dump"):
                try:
                    dumped = obj.model_dump(mode="json", exclude_none=True)
                except TypeError:
                    # fallback for different pydantic signatures
                    dumped = obj.model_dump()
            else:
                # pydantic v1 fallback
                dumped = obj.dict(exclude_none=True)
            return _normalize(dumped)
    except Exception:
        # if something unexpected happens, fall through and try other checks
        pass

    # list -> normalize items
    if isinstance(obj, list):
        return [_normalize(v) for v in obj]

    # dict -> normalize values
    if isinstance(obj, dict):
        out = {}
        for k, v in obj.items():
            out[k] = _normalize(v)
        return out

    # primitive
    return obj


def test_experimental_call_function_mocked():
    """
    Calls a view function on a contract and returns the result.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_call_function.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_call_function_response_and_rpc_call_function_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_call_function()
        else:
            params = near_jsonrpc_models.RpcCallFunctionRequest.model_validate(req_params)
            result = client.experimental_call_function(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_call_function: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_call_function: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_call_function: got={} expected={}".format(result_value, exp_resp)



def test_experimental_changes_mocked():
    """
    [Deprecated] Returns changes for a given account, contract or contract code for given block height or hash. Consider using changes instead.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_changes.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_state_changes_in_block_response_and_rpc_state_changes_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_changes()
        else:
            params = near_jsonrpc_models.RpcStateChangesInBlockByTypeRequest.model_validate(req_params)
            result = client.experimental_changes(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_changes: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_changes: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_changes: got={} expected={}".format(result_value, exp_resp)



def test_experimental_changes_in_block_mocked():
    """
    [Deprecated] Returns changes in block for given block height or hash over all transactions for all the types. Includes changes like account_touched, access_key_touched, data_touched, contract_code_touched. Consider using block_effects instead
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_changes_in_block.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_state_changes_in_block_by_type_response_and_rpc_state_changes_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_changes_in_block()
        else:
            params = near_jsonrpc_models.RpcStateChangesInBlockRequest.model_validate(req_params)
            result = client.experimental_changes_in_block(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_changes_in_block: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_changes_in_block: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_changes_in_block: got={} expected={}".format(result_value, exp_resp)



def test_experimental_congestion_level_mocked():
    """
    Queries the congestion level of a shard. More info about congestion [here](https://near.github.io/nearcore/architecture/how/receipt-congestion.html?highlight=congestion#receipt-congestion)
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_congestion_level.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_congestion_level_response_and_rpc_chunk_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_congestion_level()
        else:
            params = near_jsonrpc_models.RpcCongestionLevelRequest.model_validate(req_params)
            result = client.experimental_congestion_level(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_congestion_level: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_congestion_level: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_congestion_level: got={} expected={}".format(result_value, exp_resp)



def test_experimental_genesis_config_mocked():
    """
    [Deprecated] Get initial state and parameters for the genesis block. Consider genesis_config instead.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_genesis_config.json')
    resp_fixture = load_mock_json('json_rpc_response_for_genesis_config_and_genesis_config_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_genesis_config()
        else:
            params = near_jsonrpc_models.GenesisConfigRequest.model_validate(req_params)
            result = client.experimental_genesis_config(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_genesis_config: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_genesis_config: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_genesis_config: got={} expected={}".format(result_value, exp_resp)



def test_experimental_light_client_block_proof_mocked():
    """
    Returns the proofs for a transaction execution.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_light_client_block_proof.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_light_client_block_proof_response_and_rpc_light_client_proof_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_light_client_block_proof()
        else:
            params = near_jsonrpc_models.RpcLightClientBlockProofRequest.model_validate(req_params)
            result = client.experimental_light_client_block_proof(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_light_client_block_proof: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_light_client_block_proof: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_light_client_block_proof: got={} expected={}".format(result_value, exp_resp)



def test_experimental_light_client_proof_mocked():
    """
    Returns the proofs for a transaction execution.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_light_client_proof.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_light_client_execution_proof_response_and_rpc_light_client_proof_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_light_client_proof()
        else:
            params = near_jsonrpc_models.RpcLightClientExecutionProofRequest.model_validate(req_params)
            result = client.experimental_light_client_proof(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_light_client_proof: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_light_client_proof: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_light_client_proof: got={} expected={}".format(result_value, exp_resp)



def test_experimental_maintenance_windows_mocked():
    """
    [Deprecated] Returns the future windows for maintenance in current epoch for the specified account. In the maintenance windows, the node will not be block producer or chunk producer. Consider using maintenance_windows instead.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_maintenance_windows.json')
    resp_fixture = load_mock_json('json_rpc_response_for_array_of_range_of_uint64_and_rpc_maintenance_windows_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_maintenance_windows()
        else:
            params = near_jsonrpc_models.RpcMaintenanceWindowsRequest.model_validate(req_params)
            result = client.experimental_maintenance_windows(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_maintenance_windows: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_maintenance_windows: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_maintenance_windows: got={} expected={}".format(result_value, exp_resp)



def test_experimental_protocol_config_mocked():
    """
    A configuration that defines the protocol-level parameters such as gas/storage costs, limits, feature flags, other settings
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_protocol_config.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_protocol_config_response_and_rpc_protocol_config_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_protocol_config()
        else:
            params = near_jsonrpc_models.RpcProtocolConfigRequest.model_validate(req_params)
            result = client.experimental_protocol_config(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_protocol_config: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_protocol_config: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_protocol_config: got={} expected={}".format(result_value, exp_resp)



def test_experimental_receipt_mocked():
    """
    Fetches a receipt by its ID (as is, without a status or execution outcome)
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_receipt.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_receipt_response_and_rpc_receipt_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_receipt()
        else:
            params = near_jsonrpc_models.RpcReceiptRequest.model_validate(req_params)
            result = client.experimental_receipt(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_receipt: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_receipt: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_receipt: got={} expected={}".format(result_value, exp_resp)



def test_experimental_split_storage_info_mocked():
    """
    Contains the split storage information. More info on split storage [here](https://near-nodes.io/archival/split-storage-archival)
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_split_storage_info.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_split_storage_info_response_and_rpc_split_storage_info_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_split_storage_info()
        else:
            params = near_jsonrpc_models.RpcSplitStorageInfoRequest.model_validate(req_params)
            result = client.experimental_split_storage_info(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_split_storage_info: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_split_storage_info: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_split_storage_info: got={} expected={}".format(result_value, exp_resp)



def test_experimental_tx_status_mocked():
    """
    Queries status of a transaction by hash, returning the final transaction result and details of all receipts.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_tx_status.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_transaction_response_and_rpc_transaction_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_tx_status()
        else:
            params = near_jsonrpc_models.RpcTransactionStatusRequest.model_validate(req_params)
            result = client.experimental_tx_status(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_tx_status: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_tx_status: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_tx_status: got={} expected={}".format(result_value, exp_resp)



def test_experimental_validators_ordered_mocked():
    """
    Returns the current epoch validators ordered in the block producer order with repetition. This endpoint is solely used for bridge currently and is not intended for other external use cases.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_validators_ordered.json')
    resp_fixture = load_mock_json('json_rpc_response_for_array_of_validator_stake_view_and_rpc_validator_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_validators_ordered()
        else:
            params = near_jsonrpc_models.RpcValidatorsOrderedRequest.model_validate(req_params)
            result = client.experimental_validators_ordered(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_validators_ordered: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_validators_ordered: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_validators_ordered: got={} expected={}".format(result_value, exp_resp)



def test_experimental_view_access_key_mocked():
    """
    Returns information about a single access key for given account.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_view_access_key.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_view_access_key_response_and_rpc_view_access_key_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_view_access_key()
        else:
            params = near_jsonrpc_models.RpcViewAccessKeyRequest.model_validate(req_params)
            result = client.experimental_view_access_key(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_view_access_key: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_view_access_key: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_view_access_key: got={} expected={}".format(result_value, exp_resp)



def test_experimental_view_access_key_list_mocked():
    """
    Returns all access keys for a given account.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_view_access_key_list.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_view_access_key_list_response_and_rpc_view_access_key_list_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_view_access_key_list()
        else:
            params = near_jsonrpc_models.RpcViewAccessKeyListRequest.model_validate(req_params)
            result = client.experimental_view_access_key_list(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_view_access_key_list: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_view_access_key_list: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_view_access_key_list: got={} expected={}".format(result_value, exp_resp)



def test_experimental_view_account_mocked():
    """
    Returns information about an account for given account_id.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_view_account.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_view_account_response_and_rpc_view_account_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_view_account()
        else:
            params = near_jsonrpc_models.RpcViewAccountRequest.model_validate(req_params)
            result = client.experimental_view_account(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_view_account: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_view_account: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_view_account: got={} expected={}".format(result_value, exp_resp)



def test_experimental_view_code_mocked():
    """
    Returns the contract code (Wasm binary) deployed to the account.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_view_code.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_view_code_response_and_rpc_view_code_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_view_code()
        else:
            params = near_jsonrpc_models.RpcViewCodeRequest.model_validate(req_params)
            result = client.experimental_view_code(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_view_code: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_view_code: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_view_code: got={} expected={}".format(result_value, exp_resp)



def test_experimental_view_gas_key_mocked():
    """
    Returns information about a single gas key for given account.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_view_gas_key.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_view_gas_key_response_and_rpc_view_gas_key_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_view_gas_key()
        else:
            params = near_jsonrpc_models.RpcViewGasKeyRequest.model_validate(req_params)
            result = client.experimental_view_gas_key(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_view_gas_key: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_view_gas_key: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_view_gas_key: got={} expected={}".format(result_value, exp_resp)



def test_experimental_view_gas_key_list_mocked():
    """
    Returns all gas keys for a given account.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_view_gas_key_list.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_view_gas_key_list_response_and_rpc_view_gas_key_list_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_view_gas_key_list()
        else:
            params = near_jsonrpc_models.RpcViewGasKeyListRequest.model_validate(req_params)
            result = client.experimental_view_gas_key_list(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_view_gas_key_list: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_view_gas_key_list: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_view_gas_key_list: got={} expected={}".format(result_value, exp_resp)



def test_experimental_view_state_mocked():
    """
    Returns the state (key-value pairs) of a contract based on the key prefix.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_experimental_view_state.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_view_state_response_and_rpc_view_state_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.experimental_view_state()
        else:
            params = near_jsonrpc_models.RpcViewStateRequest.model_validate(req_params)
            result = client.experimental_view_state(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for experimental_view_state: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for experimental_view_state: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for experimental_view_state: got={} expected={}".format(result_value, exp_resp)



def test_block_mocked():
    """
    Returns block details for given height or hash
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_block.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_block_response_and_rpc_block_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.block()
        else:
            params = near_jsonrpc_models.RpcBlockRequest.model_validate(req_params)
            result = client.block(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for block: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for block: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for block: got={} expected={}".format(result_value, exp_resp)



def test_block_effects_mocked():
    """
    Returns changes in block for given block height or hash over all transactions for all the types. Includes changes like account_touched, access_key_touched, data_touched, contract_code_touched.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_block_effects.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_state_changes_in_block_by_type_response_and_rpc_state_changes_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.block_effects()
        else:
            params = near_jsonrpc_models.RpcStateChangesInBlockRequest.model_validate(req_params)
            result = client.block_effects(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for block_effects: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for block_effects: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for block_effects: got={} expected={}".format(result_value, exp_resp)



def test_broadcast_tx_async_mocked():
    """
    [Deprecated] Sends a transaction and immediately returns transaction hash. Consider using send_tx instead.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_broadcast_tx_async.json')
    resp_fixture = load_mock_json('json_rpc_response_for_crypto_hash_and_rpc_transaction_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.broadcast_tx_async()
        else:
            params = near_jsonrpc_models.RpcSendTransactionRequest.model_validate(req_params)
            result = client.broadcast_tx_async(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for broadcast_tx_async: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for broadcast_tx_async: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for broadcast_tx_async: got={} expected={}".format(result_value, exp_resp)



def test_broadcast_tx_commit_mocked():
    """
    [Deprecated] Sends a transaction and waits until transaction is fully complete. (Has a 10 second timeout). Consider using send_tx instead.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_broadcast_tx_commit.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_transaction_response_and_rpc_transaction_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.broadcast_tx_commit()
        else:
            params = near_jsonrpc_models.RpcSendTransactionRequest.model_validate(req_params)
            result = client.broadcast_tx_commit(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for broadcast_tx_commit: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for broadcast_tx_commit: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for broadcast_tx_commit: got={} expected={}".format(result_value, exp_resp)



def test_changes_mocked():
    """
    Returns changes for a given account, contract or contract code for given block height or hash.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_changes.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_state_changes_in_block_response_and_rpc_state_changes_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.changes()
        else:
            params = near_jsonrpc_models.RpcStateChangesInBlockByTypeRequest.model_validate(req_params)
            result = client.changes(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for changes: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for changes: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for changes: got={} expected={}".format(result_value, exp_resp)



def test_chunk_mocked():
    """
    Returns details of a specific chunk. You can run a block details query to get a valid chunk hash.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_chunk.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_chunk_response_and_rpc_chunk_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.chunk()
        else:
            params = near_jsonrpc_models.RpcChunkRequest.model_validate(req_params)
            result = client.chunk(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for chunk: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for chunk: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for chunk: got={} expected={}".format(result_value, exp_resp)



def test_client_config_mocked():
    """
    Queries client node configuration
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_client_config.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_client_config_response_and_rpc_client_config_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.client_config()
        else:
            params = near_jsonrpc_models.RpcClientConfigRequest.model_validate(req_params)
            result = client.client_config(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for client_config: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for client_config: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for client_config: got={} expected={}".format(result_value, exp_resp)



def test_gas_price_mocked():
    """
    Returns gas price for a specific block_height or block_hash. Using [null] will return the most recent block's gas price.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_gas_price.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_gas_price_response_and_rpc_gas_price_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.gas_price()
        else:
            params = near_jsonrpc_models.RpcGasPriceRequest.model_validate(req_params)
            result = client.gas_price(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for gas_price: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for gas_price: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for gas_price: got={} expected={}".format(result_value, exp_resp)



def test_genesis_config_mocked():
    """
    Get initial state and parameters for the genesis block
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_genesis_config.json')
    resp_fixture = load_mock_json('json_rpc_response_for_genesis_config_and_genesis_config_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.genesis_config()
        else:
            params = near_jsonrpc_models.GenesisConfigRequest.model_validate(req_params)
            result = client.genesis_config(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for genesis_config: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for genesis_config: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for genesis_config: got={} expected={}".format(result_value, exp_resp)



def test_health_mocked():
    """
    Returns the current health status of the RPC node the client connects to.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_health.json')
    resp_fixture = load_mock_json('json_rpc_response_for_nullable_rpc_health_response_and_rpc_status_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.health()
        else:
            params = near_jsonrpc_models.RpcHealthRequest.model_validate(req_params)
            result = client.health(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for health: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for health: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for health: got={} expected={}".format(result_value, exp_resp)



def test_light_client_proof_mocked():
    """
    Returns the proofs for a transaction execution.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_light_client_proof.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_light_client_execution_proof_response_and_rpc_light_client_proof_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.light_client_proof()
        else:
            params = near_jsonrpc_models.RpcLightClientExecutionProofRequest.model_validate(req_params)
            result = client.light_client_proof(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for light_client_proof: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for light_client_proof: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for light_client_proof: got={} expected={}".format(result_value, exp_resp)



def test_maintenance_windows_mocked():
    """
    Returns the future windows for maintenance in current epoch for the specified account. In the maintenance windows, the node will not be block producer or chunk producer.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_maintenance_windows.json')
    resp_fixture = load_mock_json('json_rpc_response_for_array_of_range_of_uint64_and_rpc_maintenance_windows_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.maintenance_windows()
        else:
            params = near_jsonrpc_models.RpcMaintenanceWindowsRequest.model_validate(req_params)
            result = client.maintenance_windows(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for maintenance_windows: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for maintenance_windows: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for maintenance_windows: got={} expected={}".format(result_value, exp_resp)



def test_network_info_mocked():
    """
    Queries the current state of node network connections. This includes information about active peers, transmitted data, known producers, etc.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_network_info.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_network_info_response_and_rpc_network_info_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.network_info()
        else:
            params = near_jsonrpc_models.RpcNetworkInfoRequest.model_validate(req_params)
            result = client.network_info(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for network_info: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for network_info: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for network_info: got={} expected={}".format(result_value, exp_resp)



def test_next_light_client_block_mocked():
    """
    Returns the next light client block.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_next_light_client_block.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_light_client_next_block_response_and_rpc_light_client_next_block_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.next_light_client_block()
        else:
            params = near_jsonrpc_models.RpcLightClientNextBlockRequest.model_validate(req_params)
            result = client.next_light_client_block(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for next_light_client_block: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for next_light_client_block: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for next_light_client_block: got={} expected={}".format(result_value, exp_resp)



def test_query_mocked():
    """
    This module allows you to make generic requests to the network.  The `RpcQueryRequest` struct takes in a [`BlockReference`](https://docs.rs/near-primitives/0.12.0/near_primitives/types/enum.BlockReference.html) and a [`QueryRequest`](https://docs.rs/near-primitives/0.12.0/near_primitives/views/enum.QueryRequest.html).  The `BlockReference` enum allows you to specify a block by `Finality`, `BlockId` or `SyncCheckpoint`.  The `QueryRequest` enum provides multiple variants for performing the following actions:  - View an account's details  - View a contract's code  - View the state of an account  - View the `AccessKey` of an account  - View the `AccessKeyList` of an account  - Call a function in a contract deployed on the network.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_query.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_query_response_and_rpc_query_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.query()
        else:
            params = near_jsonrpc_models.RpcQueryRequest.model_validate(req_params)
            result = client.query(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for query: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for query: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for query: got={} expected={}".format(result_value, exp_resp)



def test_send_tx_mocked():
    """
    Sends transaction. Returns the guaranteed execution status and the results the blockchain can provide at the moment.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_send_tx.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_transaction_response_and_rpc_transaction_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.send_tx()
        else:
            params = near_jsonrpc_models.RpcSendTransactionRequest.model_validate(req_params)
            result = client.send_tx(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for send_tx: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for send_tx: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for send_tx: got={} expected={}".format(result_value, exp_resp)



def test_status_mocked():
    """
    Requests the status of the connected RPC node. This includes information about sync status, nearcore node version, protocol version, the current set of validators, etc.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_status.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_status_response_and_rpc_status_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.status()
        else:
            params = near_jsonrpc_models.RpcStatusRequest.model_validate(req_params)
            result = client.status(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for status: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for status: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for status: got={} expected={}".format(result_value, exp_resp)



def test_tx_mocked():
    """
    Queries status of a transaction by hash and returns the final transaction result.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_tx.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_transaction_response_and_rpc_transaction_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.tx()
        else:
            params = near_jsonrpc_models.RpcTransactionStatusRequest.model_validate(req_params)
            result = client.tx(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for tx: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for tx: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for tx: got={} expected={}".format(result_value, exp_resp)



def test_validators_mocked():
    """
    Queries active validators on the network. Returns details and the state of validation on the blockchain.
    """

    client = NearClientSync(base_url="https://rpc.mainnet.near.org")

    req_fixture = load_mock_json('json_rpc_request_for_validators.json')
    resp_fixture = load_mock_json('json_rpc_response_for_rpc_validator_response_and_rpc_validator_error.json')

    expected_request = req_fixture
    expected_response = resp_fixture

    with respx.mock:
        if isinstance(expected_response, dict) and {'jsonrpc','id','result'}.issubset(expected_response):
            mock_response_json = expected_response
        else:
            mock_response_json = {'jsonrpc': '2.0', 'id': 'test', 'result': expected_response}
        route = respx.post("https://rpc.mainnet.near.org").mock(return_value=Response(200, json=mock_response_json))

        req = expected_request
        if isinstance(req, dict) and 'params' in req:
            req_params = req['params']
        else:
            req_params = req

        if req_params is None:
            result = client.validators()
        else:
            params = near_jsonrpc_models.RpcValidatorRequest.model_validate(req_params)
            result = client.validators(params=params)

        assert route.called
        assert len(route.calls) >= 1
        sent = None
        req_obj = route.calls[0].request
        body = getattr(req_obj, 'content', None)
        if body is None:
            try:
                body = req_obj.read()
            except Exception:
                body = None
        if isinstance(body, bytes):
            try:
                body = body.decode('utf-8')
            except Exception:
                body = body.decode('latin-1')
        if isinstance(body, str) and body.strip():
            try:
                sent = json.loads(body)
            except Exception:
                sent = None

        if isinstance(sent, dict):
            sent_params = sent.get('params')
            exp = req_params
            if isinstance(expected_request, dict) and 'params' in expected_request:
                exp = expected_request.get('params')
            if exp is None:
                assert ('params' not in sent) or (sent.get('params') is None), "params mismatch for validators: expected None but found present"
            else:
                assert sent_params == exp, "params mismatch for validators: sent={} expected={}".format(sent_params, exp)

        exp_resp = expected_response
        if isinstance(exp_resp, dict) and 'result' in exp_resp:
            exp_resp = exp_resp['result']

        result_value = _normalize(result)

        assert result_value == exp_resp, "result mismatch for validators: got={} expected={}".format(result_value, exp_resp)



from near_jsonrpc_models.rpc_receipt_request import RpcReceiptRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalReceipt(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_receipt']
    params: RpcReceiptRequest



"""A result returned by contract method"""

from pydantic import BaseModel
from pydantic import conint
from typing import List


class CallResult(BaseModel):
    logs: List[str]
    result: List[conint(ge=0, le=255)]



"""Describes information about the current epoch validator"""

from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.near_token import NearToken
from near_jsonrpc_models.public_key import PublicKey
from near_jsonrpc_models.shard_id import ShardId
from pydantic import BaseModel
from pydantic import Field
from pydantic import conint
from typing import List


class CurrentEpochValidatorInfo(BaseModel):
    account_id: AccountId
    is_slashed: bool
    num_expected_blocks: conint(ge=0, le=18446744073709551615)
    num_expected_chunks: conint(ge=0, le=18446744073709551615) = 0
    # Number of chunks this validator was expected to produce in each shard.
    # Each entry in the array corresponds to the shard in the `shards_produced` array.
    num_expected_chunks_per_shard: List[conint(ge=0, le=18446744073709551615)] = Field(default_factory=lambda: [])
    num_expected_endorsements: conint(ge=0, le=18446744073709551615) = 0
    # Number of chunks this validator was expected to validate and endorse in each shard.
    # Each entry in the array corresponds to the shard in the `shards_endorsed` array.
    num_expected_endorsements_per_shard: List[conint(ge=0, le=18446744073709551615)] = Field(default_factory=lambda: [])
    num_produced_blocks: conint(ge=0, le=18446744073709551615)
    num_produced_chunks: conint(ge=0, le=18446744073709551615) = 0
    num_produced_chunks_per_shard: List[conint(ge=0, le=18446744073709551615)] = Field(default_factory=lambda: [])
    num_produced_endorsements: conint(ge=0, le=18446744073709551615) = 0
    num_produced_endorsements_per_shard: List[conint(ge=0, le=18446744073709551615)] = Field(default_factory=lambda: [])
    public_key: PublicKey
    # Shards this validator is assigned to as chunk producer in the current epoch.
    shards: List[ShardId]
    # Shards this validator is assigned to as chunk validator in the current epoch.
    shards_endorsed: List[ShardId] = Field(default_factory=lambda: [ShardId(**item) for item in []])
    stake: NearToken



from near_jsonrpc_models.epoch_id import EpochId
from pydantic import BaseModel
from pydantic import RootModel
from pydantic import conint
from pydantic import conlist
from typing import List
from typing import Literal
from typing import Union


class RpcStatusErrorNodeIsSyncing(BaseModel):
    name: Literal['NODE_IS_SYNCING']

class RpcStatusErrorNoNewBlocksInfo(BaseModel):
    elapsed: conlist(conint(ge=0, le=18446744073709551615), min_length=2, max_length=2)

class RpcStatusErrorNoNewBlocks(BaseModel):
    info: RpcStatusErrorNoNewBlocksInfo
    name: Literal['NO_NEW_BLOCKS']

class RpcStatusErrorEpochOutOfBoundsInfo(BaseModel):
    epoch_id: EpochId

class RpcStatusErrorEpochOutOfBounds(BaseModel):
    info: RpcStatusErrorEpochOutOfBoundsInfo
    name: Literal['EPOCH_OUT_OF_BOUNDS']

class RpcStatusErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcStatusErrorInternalError(BaseModel):
    info: RpcStatusErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcStatusError(RootModel[Union[RpcStatusErrorNodeIsSyncing, RpcStatusErrorNoNewBlocks, RpcStatusErrorEpochOutOfBounds, RpcStatusErrorInternalError]]):
    pass




from near_jsonrpc_models.access_key_permission_view import AccessKeyPermissionView
from near_jsonrpc_models.near_token import NearToken
from pydantic import BaseModel
from pydantic import conint
from typing import List


class GasKeyView(BaseModel):
    balance: NearToken
    nonces: List[conint(ge=0, le=18446744073709551615)]
    num_nonces: conint(ge=0, le=4294967295)
    permission: AccessKeyPermissionView



from near_jsonrpc_models.rpc_query_request import RpcQueryRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForQuery(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['query']
    params: RpcQueryRequest



"""View that preserves JSON format of the runtime config."""

from near_jsonrpc_models.account_creation_config_view import AccountCreationConfigView
from near_jsonrpc_models.congestion_control_config_view import CongestionControlConfigView
from near_jsonrpc_models.dynamic_resharding_config_view import DynamicReshardingConfigView
from near_jsonrpc_models.near_token import NearToken
from near_jsonrpc_models.runtime_fees_config_view import RuntimeFeesConfigView
from near_jsonrpc_models.vmconfig_view import VMConfigView
from near_jsonrpc_models.witness_config_view import WitnessConfigView
from pydantic import BaseModel
from pydantic import Field


class RuntimeConfigView(BaseModel):
    # Config that defines rules for account creation.
    account_creation_config: AccountCreationConfigView = None
    # The configuration for congestion control.
    congestion_control_config: CongestionControlConfigView = None
    # Configuration for dynamic resharding feature.
    dynamic_resharding_config: DynamicReshardingConfigView = Field(default_factory=lambda: DynamicReshardingConfigView(**{'max_number_of_shards': 999999999999999, 'memory_usage_threshold': 999999999999999, 'min_child_memory_usage': 999999999999999, 'min_epochs_between_resharding': 999999999999999}))
    # Amount of yN per byte required to have on the account.  See
    # <https://nomicon.io/Economics/Economics.html#state-stake> for details.
    storage_amount_per_byte: NearToken = None
    # Costs of different actions that need to be performed when sending and
    # processing transaction and receipts.
    transaction_costs: RuntimeFeesConfigView = None
    # Config of wasm operations.
    wasm_config: VMConfigView = None
    # Configuration specific to ChunkStateWitness.
    witness_config: WitnessConfigView = None



from near_jsonrpc_models.account_data_view import AccountDataView
from near_jsonrpc_models.known_producer_view import KnownProducerView
from near_jsonrpc_models.peer_info_view import PeerInfoView
from near_jsonrpc_models.public_key import PublicKey
from pydantic import BaseModel
from pydantic import conint
from typing import List


class NetworkInfoView(BaseModel):
    connected_peers: List[PeerInfoView]
    known_producers: List[KnownProducerView]
    num_connected_peers: conint(ge=0, le=4294967295)
    peer_max_count: conint(ge=0, le=4294967295)
    tier1_accounts_data: List[AccountDataView]
    tier1_accounts_keys: List[PublicKey]
    tier1_connections: List[PeerInfoView]



"""Configuration specific to ChunkStateWitness."""

from pydantic import BaseModel
from pydantic import conint


class WitnessConfigView(BaseModel):
    # Maximum size of transactions contained inside ChunkStateWitness.
    # 
    # A witness contains transactions from both the previous chunk and the current one.
    # This parameter limits the sum of sizes of transactions from both of those chunks.
    combined_transactions_size_limit: conint(ge=0, le=4294967295) = None
    # Size limit for storage proof generated while executing receipts in a chunk.
    # After this limit is reached we defer execution of any new receipts.
    main_storage_proof_size_soft_limit: conint(ge=0, le=18446744073709551615) = None
    # Soft size limit of storage proof used to validate new transactions in ChunkStateWitness.
    new_transactions_validation_state_size_soft_limit: conint(ge=0, le=18446744073709551615) = None



"""NEAR Account Identifier.

This is a unique, syntactically valid, human-readable account identifier on the NEAR network.

[See the crate-level docs for information about validation.](index.html#account-id-rules)

Also see [Error kind precedence](AccountId#error-kind-precedence).

## Examples

```
use near_account_id::AccountId;

let alice: AccountId = "alice.near".parse().unwrap();

assert!("ƒelicia.near".parse::<AccountId>().is_err()); // (ƒ is not f)
```"""

from pydantic import BaseModel
from pydantic import RootModel


class AccountId(RootModel[str]):
    pass




from near_jsonrpc_models.block_status_view import BlockStatusView
from near_jsonrpc_models.catchup_status_view import CatchupStatusView
from near_jsonrpc_models.network_info_view import NetworkInfoView
from pydantic import BaseModel
from pydantic import conint
from typing import List


class DetailedDebugStatus(BaseModel):
    block_production_delay_millis: conint(ge=0, le=18446744073709551615)
    catchup_status: List[CatchupStatusView]
    current_head_status: BlockStatusView
    current_header_head_status: BlockStatusView
    network_info: NetworkInfoView
    sync_status: str



"""Describes limits for VM and Runtime.
TODO #4139: consider switching to strongly-typed wrappers instead of raw quantities"""

from near_jsonrpc_models.account_id_validity_rules_version import AccountIdValidityRulesVersion
from near_jsonrpc_models.near_gas import NearGas
from pydantic import BaseModel
from pydantic import Field
from pydantic import conint


class LimitConfig(BaseModel):
    # Whether to enforce account_id well-formed-ness where it wasn't enforced
    # historically.
    account_id_validity_rules_version: AccountIdValidityRulesVersion = Field(default_factory=lambda: AccountIdValidityRulesVersion(0))
    # The initial number of memory pages.
    # NOTE: It's not a limiter itself, but it's a value we use for initial_memory_pages.
    initial_memory_pages: conint(ge=0, le=4294967295) = None
    # Max number of actions per receipt.
    max_actions_per_receipt: conint(ge=0, le=18446744073709551615) = None
    # Max length of arguments in a function call action.
    max_arguments_length: conint(ge=0, le=18446744073709551615) = None
    # Max contract size
    max_contract_size: conint(ge=0, le=18446744073709551615) = None
    # If present, stores max number of elements in a single contract's table
    max_elements_per_contract_table: conint(ge=0, le=4294967295) | None = None
    # If present, stores max number of functions in one contract
    max_functions_number_per_contract: conint(ge=0, le=18446744073709551615) | None = None
    # Max amount of gas that can be used, excluding gas attached to promises.
    max_gas_burnt: NearGas = None
    # Max length of any method name (without terminating character).
    max_length_method_name: conint(ge=0, le=18446744073709551615) = None
    # Max length of returned data
    max_length_returned_data: conint(ge=0, le=18446744073709551615) = None
    # Max storage key size
    max_length_storage_key: conint(ge=0, le=18446744073709551615) = None
    # Max storage value size
    max_length_storage_value: conint(ge=0, le=18446744073709551615) = None
    # If present, stores max number of locals declared globally in one contract
    max_locals_per_contract: conint(ge=0, le=18446744073709551615) | None = None
    # What is the maximal memory pages amount is allowed to have for a contract.
    max_memory_pages: conint(ge=0, le=4294967295) = None
    # Max total length of all method names (including terminating character) for a function call
    # permission access key.
    max_number_bytes_method_names: conint(ge=0, le=18446744073709551615) = None
    # Max number of input data dependencies
    max_number_input_data_dependencies: conint(ge=0, le=18446744073709551615) = None
    # Maximum number of log entries.
    max_number_logs: conint(ge=0, le=18446744073709551615) = None
    # Maximum number of registers that can be used simultaneously.
    # 
    # Note that due to an implementation quirk [read: a bug] in VMLogic, if we
    # have this number of registers, no subsequent writes to the registers
    # will succeed even if they replace an existing register.
    max_number_registers: conint(ge=0, le=18446744073709551615) = None
    # Max number of promises that a function call can create
    max_promises_per_function_call_action: conint(ge=0, le=18446744073709551615) = None
    # Max receipt size
    max_receipt_size: conint(ge=0, le=18446744073709551615) = None
    # Maximum number of bytes that can be stored in a single register.
    max_register_size: conint(ge=0, le=18446744073709551615) = None
    # How tall the stack is allowed to grow?
    # 
    # See <https://wiki.parity.io/WebAssembly-StackHeight> to find out how the stack frame cost
    # is calculated.
    max_stack_height: conint(ge=0, le=4294967295) = None
    # If present, stores max number of tables declared globally in one contract
    max_tables_per_contract: conint(ge=0, le=4294967295) | None = None
    # Maximum total length in bytes of all log messages.
    max_total_log_length: conint(ge=0, le=18446744073709551615) = None
    # Max total prepaid gas for all function call actions per receipt.
    max_total_prepaid_gas: NearGas = None
    # Max transaction size
    max_transaction_size: conint(ge=0, le=18446744073709551615) = None
    # Maximum number of bytes for payload passed over a yield resume.
    max_yield_payload_size: conint(ge=0, le=18446744073709551615) = None
    # Hard limit on the size of storage proof generated while executing a single receipt.
    per_receipt_storage_proof_size_limit: conint(ge=0, le=4294967295) = None
    # Limit of memory used by registers.
    registers_memory_limit: conint(ge=0, le=18446744073709551615) = None
    # Number of blocks after which a yielded promise times out.
    yield_timeout_length_in_blocks: conint(ge=0, le=18446744073709551615) = None



from near_jsonrpc_client.api_methods_async import APIMixinAsync
from near_jsonrpc_client.api_methods_sync import APIMixinSync
from .base_client import NearBaseClientAsync, NearBaseClientSync


class NearClientAsync(NearBaseClientAsync, APIMixinAsync):
    """NearClientAsync with generated API methods mixed in."""
    pass


class NearClientSync(NearBaseClientSync, APIMixinSync):
    """NearClientSync with generated API methods mixed in."""
    pass



from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.near_token import NearToken
from near_jsonrpc_models.public_key import PublicKey
from near_jsonrpc_models.shard_id import ShardId
from pydantic import BaseModel
from typing import List


class NextEpochValidatorInfo(BaseModel):
    account_id: AccountId
    public_key: PublicKey
    shards: List[ShardId]
    stake: NearToken



"""A versioned struct that contains all information needed to assign accounts to shards.

Because of re-sharding, the chain may use different shard layout to split shards at different
times. Currently, `ShardLayout` is stored as part of `EpochConfig`, which is generated each
epoch given the epoch protocol version. In mainnet/testnet, we use two shard layouts since
re-sharding has only happened once. It is stored as part of genesis config, see
default_simple_nightshade_shard_layout() Below is an overview for some important
functionalities of ShardLayout interface."""

from near_jsonrpc_models.shard_layout_v0 import ShardLayoutV0
from near_jsonrpc_models.shard_layout_v1 import ShardLayoutV1
from near_jsonrpc_models.shard_layout_v2 import ShardLayoutV2
from near_jsonrpc_models.shard_layout_v3 import ShardLayoutV3
from near_jsonrpc_models.strict_model import StrictBaseModel
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class ShardLayoutV0Option(StrictBaseModel):
    V0: ShardLayoutV0

class ShardLayoutV1Option(StrictBaseModel):
    V1: ShardLayoutV1

class ShardLayoutV2Option(StrictBaseModel):
    V2: ShardLayoutV2

class ShardLayoutV3Option(StrictBaseModel):
    V3: ShardLayoutV3

class ShardLayout(RootModel[Union[ShardLayoutV0Option, ShardLayoutV1Option, ShardLayoutV2Option, ShardLayoutV3Option]]):
    pass




from near_jsonrpc_models.error_wrapper_for_rpc_client_config_error import ErrorWrapperForRpcClientConfigError
from near_jsonrpc_models.rpc_client_config_response import RpcClientConfigResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcClientConfigResponseAndRpcClientConfigErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcClientConfigResponse

class JsonRpcResponseForRpcClientConfigResponseAndRpcClientConfigErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcClientConfigError

class JsonRpcResponseForRpcClientConfigResponseAndRpcClientConfigError(RootModel[Union[JsonRpcResponseForRpcClientConfigResponseAndRpcClientConfigErrorResult, JsonRpcResponseForRpcClientConfigResponseAndRpcClientConfigErrorError]]):
    pass




"""Error that can occur while preparing or executing Wasm smart-contract.Serialization: Error happened while serializing the module.Deserialization: Error happened while deserializing the module.InternalMemoryDeclared: Internal memory declaration has been found in the module.GasInstrumentation: Gas instrumentation failed.

This most likely indicates the module isn't valid.StackHeightInstrumentation: Stack instrumentation failed.

This  most likely indicates the module isn't valid.Instantiate: Error happened during instantiation.

This might indicate that `start` function trapped, or module isn't
instantiable and/or un-linkable.Memory: Error creating memory.TooManyFunctions: Contract contains too many functions.TooManyLocals: Contract contains too many locals.TooManyTables: Contract contains too many tables.TooManyTableElements: Contract contains too many table elements."""

from pydantic import RootModel
from typing import Literal


class PrepareError(RootModel[Literal['Serialization', 'Deserialization', 'InternalMemoryDeclared', 'GasInstrumentation', 'StackHeightInstrumentation', 'Instantiate', 'Memory', 'TooManyFunctions', 'TooManyLocals', 'TooManyTables', 'TooManyTableElements']]):
    pass




from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.block_reference import BlockReference
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.public_key import PublicKey
from pydantic import BaseModel
from pydantic import RootModel
from pydantic import conint
from typing import Literal
from typing import Union


class RpcViewGasKeyErrorUnknownBlockInfo(BaseModel):
    block_reference: BlockReference

class RpcViewGasKeyErrorUnknownBlock(BaseModel):
    info: RpcViewGasKeyErrorUnknownBlockInfo
    name: Literal['UNKNOWN_BLOCK']

class RpcViewGasKeyErrorInvalidAccountInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    requested_account_id: AccountId

class RpcViewGasKeyErrorInvalidAccount(BaseModel):
    info: RpcViewGasKeyErrorInvalidAccountInfo
    name: Literal['INVALID_ACCOUNT']

class RpcViewGasKeyErrorUnknownAccountInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    requested_account_id: AccountId

class RpcViewGasKeyErrorUnknownAccount(BaseModel):
    info: RpcViewGasKeyErrorUnknownAccountInfo
    name: Literal['UNKNOWN_ACCOUNT']

class RpcViewGasKeyErrorUnknownGasKeyInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    public_key: PublicKey

class RpcViewGasKeyErrorUnknownGasKey(BaseModel):
    info: RpcViewGasKeyErrorUnknownGasKeyInfo
    name: Literal['UNKNOWN_GAS_KEY']

class RpcViewGasKeyErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcViewGasKeyErrorInternalError(BaseModel):
    info: RpcViewGasKeyErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcViewGasKeyError(RootModel[Union[RpcViewGasKeyErrorUnknownBlock, RpcViewGasKeyErrorInvalidAccount, RpcViewGasKeyErrorUnknownAccount, RpcViewGasKeyErrorUnknownGasKey, RpcViewGasKeyErrorInternalError]]):
    pass




from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.peer_id import PeerId
from pydantic import BaseModel


class RpcPeerInfo(BaseModel):
    account_id: AccountId | None = None
    addr: str | None = None
    id: PeerId



from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.chunk_header_view import ChunkHeaderView
from near_jsonrpc_models.receipt_view import ReceiptView
from near_jsonrpc_models.signed_transaction_view import SignedTransactionView
from pydantic import BaseModel
from typing import List


class RpcChunkResponse(BaseModel):
    author: AccountId
    header: ChunkHeaderView
    receipts: List[ReceiptView]
    transactions: List[SignedTransactionView]



from near_jsonrpc_models.internal_error import InternalError
from near_jsonrpc_models.rpc_call_function_error import RpcCallFunctionError
from near_jsonrpc_models.rpc_request_validation_error_kind import RpcRequestValidationErrorKind
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class ErrorWrapperForRpcCallFunctionErrorRequestValidationError(BaseModel):
    cause: RpcRequestValidationErrorKind
    name: Literal['REQUEST_VALIDATION_ERROR']

class ErrorWrapperForRpcCallFunctionErrorHandlerError(BaseModel):
    cause: RpcCallFunctionError
    name: Literal['HANDLER_ERROR']

class ErrorWrapperForRpcCallFunctionErrorInternalError(BaseModel):
    cause: InternalError
    name: Literal['INTERNAL_ERROR']

class ErrorWrapperForRpcCallFunctionError(RootModel[Union[ErrorWrapperForRpcCallFunctionErrorRequestValidationError, ErrorWrapperForRpcCallFunctionErrorHandlerError, ErrorWrapperForRpcCallFunctionErrorInternalError]]):
    pass




from near_jsonrpc_models.internal_error import InternalError
from near_jsonrpc_models.rpc_client_config_error import RpcClientConfigError
from near_jsonrpc_models.rpc_request_validation_error_kind import RpcRequestValidationErrorKind
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class ErrorWrapperForRpcClientConfigErrorRequestValidationError(BaseModel):
    cause: RpcRequestValidationErrorKind
    name: Literal['REQUEST_VALIDATION_ERROR']

class ErrorWrapperForRpcClientConfigErrorHandlerError(BaseModel):
    cause: RpcClientConfigError
    name: Literal['HANDLER_ERROR']

class ErrorWrapperForRpcClientConfigErrorInternalError(BaseModel):
    cause: InternalError
    name: Literal['INTERNAL_ERROR']

class ErrorWrapperForRpcClientConfigError(RootModel[Union[ErrorWrapperForRpcClientConfigErrorRequestValidationError, ErrorWrapperForRpcClientConfigErrorHandlerError, ErrorWrapperForRpcClientConfigErrorInternalError]]):
    pass




"""This type is used to mark keys (arrays of bytes) that are queried from store.

NOTE: Currently, this type is only used in the view_client and RPC to be able to transparently
pretty-serialize the bytes arrays as base64-encoded strings (see `serialize.rs`)."""

from pydantic import BaseModel
from pydantic import RootModel


class StoreKey(RootModel[str]):
    pass




from near_jsonrpc_models.access_key_permission import AccessKeyPermission
from near_jsonrpc_models.public_key import PublicKey
from pydantic import BaseModel
from pydantic import conint


class AddGasKeyAction(BaseModel):
    num_nonces: conint(ge=0, le=4294967295)
    permission: AccessKeyPermission
    public_key: PublicKey



from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.crypto_hash import CryptoHash
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class RpcLightClientExecutionProofRequestTransaction(BaseModel):
    light_client_head: CryptoHash
    sender_id: AccountId
    transaction_hash: CryptoHash
    type: Literal['transaction']

class RpcLightClientExecutionProofRequestReceipt(BaseModel):
    light_client_head: CryptoHash
    receipt_id: CryptoHash
    receiver_id: AccountId
    type: Literal['receipt']

class RpcLightClientExecutionProofRequest(RootModel[Union[RpcLightClientExecutionProofRequestTransaction, RpcLightClientExecutionProofRequestReceipt]]):
    pass




from near_jsonrpc_models.rpc_protocol_config_request import RpcProtocolConfigRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalProtocolConfig(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_protocol_config']
    params: RpcProtocolConfigRequest



"""`ShardUId` is a unique representation for shards from different shard layouts.

Comparing to `ShardId`, which is just an ordinal number ranging from 0 to NUM_SHARDS-1,
`ShardUId` provides a way to unique identify shards when shard layouts may change across epochs.
This is important because we store states indexed by shards in our database, so we need a
way to unique identify shard even when shards change across epochs.
Another difference between `ShardUId` and `ShardId` is that `ShardUId` should only exist in
a node's internal state while `ShardId` can be exposed to outside APIs and used in protocol
level information (for example, `ShardChunkHeader` contains `ShardId` instead of `ShardUId`)"""

from pydantic import BaseModel
from pydantic import conint


class ShardUId(BaseModel):
    shard_id: conint(ge=0, le=4294967295)
    version: conint(ge=0, le=4294967295)



"""A view of the account"""

from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.near_token import NearToken
from pydantic import BaseModel
from pydantic import conint


class AccountView(BaseModel):
    amount: NearToken
    code_hash: CryptoHash
    global_contract_account_id: AccountId | None = None
    global_contract_hash: CryptoHash | None = None
    locked: NearToken
    # TODO(2271): deprecated.
    storage_paid_at: conint(ge=0, le=18446744073709551615) = 0
    storage_usage: conint(ge=0, le=18446744073709551615)



from near_jsonrpc_models.block_id import BlockId
from near_jsonrpc_models.epoch_id import EpochId
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class RpcValidatorRequestLatest(RootModel[Literal['latest']]):
    pass

class RpcValidatorRequestEpochId(BaseModel):
    epoch_id: EpochId

class RpcValidatorRequestBlockId(BaseModel):
    block_id: BlockId

class RpcValidatorRequest(RootModel[Union[RpcValidatorRequestLatest, RpcValidatorRequestEpochId, RpcValidatorRequestBlockId]]):
    pass




from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.direction import Direction
from pydantic import BaseModel


class MerklePathItem(BaseModel):
    direction: Direction
    hash: CryptoHash



from near_jsonrpc_models.internal_error import InternalError
from near_jsonrpc_models.rpc_request_validation_error_kind import RpcRequestValidationErrorKind
from near_jsonrpc_models.rpc_view_code_error import RpcViewCodeError
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class ErrorWrapperForRpcViewCodeErrorRequestValidationError(BaseModel):
    cause: RpcRequestValidationErrorKind
    name: Literal['REQUEST_VALIDATION_ERROR']

class ErrorWrapperForRpcViewCodeErrorHandlerError(BaseModel):
    cause: RpcViewCodeError
    name: Literal['HANDLER_ERROR']

class ErrorWrapperForRpcViewCodeErrorInternalError(BaseModel):
    cause: InternalError
    name: Literal['INTERNAL_ERROR']

class ErrorWrapperForRpcViewCodeError(RootModel[Union[ErrorWrapperForRpcViewCodeErrorRequestValidationError, ErrorWrapperForRpcViewCodeErrorHandlerError, ErrorWrapperForRpcViewCodeErrorInternalError]]):
    pass




"""Defines permissions for AccessKey"""

from near_jsonrpc_models.function_call_permission import FunctionCallPermission
from near_jsonrpc_models.strict_model import StrictBaseModel
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class AccessKeyPermissionFunctionCall(StrictBaseModel):
    FunctionCall: FunctionCallPermission

"""Grants full access to the account.
NOTE: It's used to replace account-level public keys."""
class AccessKeyPermissionFullAccess(RootModel[Literal['FullAccess']]):
    pass

class AccessKeyPermission(RootModel[Union[AccessKeyPermissionFunctionCall, AccessKeyPermissionFullAccess]]):
    pass




from near_jsonrpc_models.rpc_network_info_request import RpcNetworkInfoRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForNetworkInfo(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['network_info']
    params: RpcNetworkInfoRequest



from pydantic import BaseModel
from pydantic import RootModel
from typing import Any
from typing import Dict
from typing import Literal
from typing import Union


class RpcStateChangesErrorUnknownBlock(BaseModel):
    info: Dict[str, Any]
    name: Literal['UNKNOWN_BLOCK']

class RpcStateChangesErrorNotSyncedYet(BaseModel):
    name: Literal['NOT_SYNCED_YET']

class RpcStateChangesErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcStateChangesErrorInternalError(BaseModel):
    info: RpcStateChangesErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcStateChangesError(RootModel[Union[RpcStateChangesErrorUnknownBlock, RpcStateChangesErrorNotSyncedYet, RpcStateChangesErrorInternalError]]):
    pass




from .client import NearClientAsync, NearClientSync
from .transport import HttpTransportAsync, HttpTransportSync
from .errors import (
    ClientError,
    TransportError,
    HttpError,
    RpcError,
    RequestTimeoutError
)

__all__ = [
    # Main clients
    "NearClientAsync",
    "NearClientSync",

    # Transport

    "HttpTransportAsync",
    "HttpTransportSync",

    # Errors
    "ClientError",
    "TransportError",
    "HttpError",
    "RpcError",
    "RequestTimeoutError"
]



from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class RpcNetworkInfoErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcNetworkInfoErrorInternalError(BaseModel):
    info: RpcNetworkInfoErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcNetworkInfoError(RootModel[Union[RpcNetworkInfoErrorInternalError]]):
    pass




from near_jsonrpc_models.internal_error import InternalError
from near_jsonrpc_models.rpc_request_validation_error_kind import RpcRequestValidationErrorKind
from near_jsonrpc_models.rpc_view_account_error import RpcViewAccountError
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class ErrorWrapperForRpcViewAccountErrorRequestValidationError(BaseModel):
    cause: RpcRequestValidationErrorKind
    name: Literal['REQUEST_VALIDATION_ERROR']

class ErrorWrapperForRpcViewAccountErrorHandlerError(BaseModel):
    cause: RpcViewAccountError
    name: Literal['HANDLER_ERROR']

class ErrorWrapperForRpcViewAccountErrorInternalError(BaseModel):
    cause: InternalError
    name: Literal['INTERNAL_ERROR']

class ErrorWrapperForRpcViewAccountError(RootModel[Union[ErrorWrapperForRpcViewAccountErrorRequestValidationError, ErrorWrapperForRpcViewAccountErrorHandlerError, ErrorWrapperForRpcViewAccountErrorInternalError]]):
    pass




from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.shard_id import ShardId
from pydantic import BaseModel
from pydantic import conint
from typing import List


class ShardLayoutV1(BaseModel):
    # The boundary accounts are the accounts on boundaries between shards.
    # Each shard contains a range of accounts from one boundary account to
    # another - or the smallest or largest account possible. The total
    # number of shards is equal to the number of boundary accounts plus 1.
    boundary_accounts: List[AccountId]
    # Maps shards from the last shard layout to shards that it splits to in this shard layout,
    # Useful for constructing states for the shards.
    # None for the genesis shard layout
    shards_split_map: List[List[ShardId]] | None = None
    # Maps shard in this shard layout to their parent shard
    # Since shard_ids always range from 0 to num_shards - 1, we use vec instead of a hashmap
    to_parent_shard_map: List[ShardId] | None = None
    # Version of the shard layout, this is useful for uniquely identify the shard layout
    version: conint(ge=0, le=4294967295)



from near_jsonrpc_models.error_wrapper_for_rpc_network_info_error import ErrorWrapperForRpcNetworkInfoError
from near_jsonrpc_models.rpc_network_info_response import RpcNetworkInfoResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcNetworkInfoResponseAndRpcNetworkInfoErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcNetworkInfoResponse

class JsonRpcResponseForRpcNetworkInfoResponseAndRpcNetworkInfoErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcNetworkInfoError

class JsonRpcResponseForRpcNetworkInfoResponseAndRpcNetworkInfoError(RootModel[Union[JsonRpcResponseForRpcNetworkInfoResponseAndRpcNetworkInfoErrorResult, JsonRpcResponseForRpcNetworkInfoResponseAndRpcNetworkInfoErrorError]]):
    pass




from near_jsonrpc_models.internal_error import InternalError
from near_jsonrpc_models.rpc_network_info_error import RpcNetworkInfoError
from near_jsonrpc_models.rpc_request_validation_error_kind import RpcRequestValidationErrorKind
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class ErrorWrapperForRpcNetworkInfoErrorRequestValidationError(BaseModel):
    cause: RpcRequestValidationErrorKind
    name: Literal['REQUEST_VALIDATION_ERROR']

class ErrorWrapperForRpcNetworkInfoErrorHandlerError(BaseModel):
    cause: RpcNetworkInfoError
    name: Literal['HANDLER_ERROR']

class ErrorWrapperForRpcNetworkInfoErrorInternalError(BaseModel):
    cause: InternalError
    name: Literal['INTERNAL_ERROR']

class ErrorWrapperForRpcNetworkInfoError(RootModel[Union[ErrorWrapperForRpcNetworkInfoErrorRequestValidationError, ErrorWrapperForRpcNetworkInfoErrorHandlerError, ErrorWrapperForRpcNetworkInfoErrorInternalError]]):
    pass




from pydantic import BaseModel
from pydantic import conint


class SyncConcurrency(BaseModel):
    # Maximum number of "apply parts" tasks that can be performed in parallel.
    # This is a very disk-heavy task and therefore we set this to a low limit,
    # or else the rocksdb contention makes the whole server freeze up.
    apply: conint(ge=0, le=255) = None
    # Maximum number of "apply parts" tasks that can be performed in parallel
    # during catchup. We set this to a very low value to avoid overloading the
    # node while it is still performing normal tasks.
    apply_during_catchup: conint(ge=0, le=255) = None
    # Maximum number of outstanding requests for decentralized state sync.
    peer_downloads: conint(ge=0, le=255) = None
    # The maximum parallelism to use per shard. This is mostly for fairness, because
    # the actual rate limiting is done by the TaskTrackers, but this is useful for
    # balancing the shards a little.
    per_shard: conint(ge=0, le=255) = None



from pydantic import BaseModel
from pydantic import RootModel
from typing import Any
from typing import Dict
from typing import Literal
from typing import Union


class RpcGasPriceErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcGasPriceErrorInternalError(BaseModel):
    info: RpcGasPriceErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcGasPriceErrorUnknownBlock(BaseModel):
    info: Dict[str, Any]
    name: Literal['UNKNOWN_BLOCK']

class RpcGasPriceError(RootModel[Union[RpcGasPriceErrorInternalError, RpcGasPriceErrorUnknownBlock]]):
    pass




from near_jsonrpc_models.error_wrapper_for_rpc_view_gas_key_error import ErrorWrapperForRpcViewGasKeyError
from near_jsonrpc_models.rpc_view_gas_key_response import RpcViewGasKeyResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcViewGasKeyResponseAndRpcViewGasKeyErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcViewGasKeyResponse

class JsonRpcResponseForRpcViewGasKeyResponseAndRpcViewGasKeyErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcViewGasKeyError

class JsonRpcResponseForRpcViewGasKeyResponseAndRpcViewGasKeyError(RootModel[Union[JsonRpcResponseForRpcViewGasKeyResponseAndRpcViewGasKeyErrorResult, JsonRpcResponseForRpcViewGasKeyResponseAndRpcViewGasKeyErrorError]]):
    pass




from near_jsonrpc_models.internal_error import InternalError
from near_jsonrpc_models.rpc_request_validation_error_kind import RpcRequestValidationErrorKind
from near_jsonrpc_models.rpc_validator_error import RpcValidatorError
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class ErrorWrapperForRpcValidatorErrorRequestValidationError(BaseModel):
    cause: RpcRequestValidationErrorKind
    name: Literal['REQUEST_VALIDATION_ERROR']

class ErrorWrapperForRpcValidatorErrorHandlerError(BaseModel):
    cause: RpcValidatorError
    name: Literal['HANDLER_ERROR']

class ErrorWrapperForRpcValidatorErrorInternalError(BaseModel):
    cause: InternalError
    name: Literal['INTERNAL_ERROR']

class ErrorWrapperForRpcValidatorError(RootModel[Union[ErrorWrapperForRpcValidatorErrorRequestValidationError, ErrorWrapperForRpcValidatorErrorHandlerError, ErrorWrapperForRpcValidatorErrorInternalError]]):
    pass




from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.strict_model import StrictBaseModel
from near_jsonrpc_models.tx_execution_error import TxExecutionError
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


"""The execution is pending or unknown."""
class ExecutionStatusViewUnknown(RootModel[Literal['Unknown']]):
    pass

class ExecutionStatusViewFailure(StrictBaseModel):
    """The execution has failed."""
    Failure: TxExecutionError

class ExecutionStatusViewSuccessValue(StrictBaseModel):
    """The final action succeeded and returned some value or an empty vec encoded in base64."""
    SuccessValue: str

class ExecutionStatusViewSuccessReceiptId(StrictBaseModel):
    """The final action of the receipt returned a promise or the signed transaction was converted
to a receipt. Contains the receipt_id of the generated receipt."""
    SuccessReceiptId: CryptoHash

class ExecutionStatusView(RootModel[Union[ExecutionStatusViewUnknown, ExecutionStatusViewFailure, ExecutionStatusViewSuccessValue, ExecutionStatusViewSuccessReceiptId]]):
    pass




from near_jsonrpc_models.block_header_inner_lite_view import BlockHeaderInnerLiteView
from near_jsonrpc_models.crypto_hash import CryptoHash
from pydantic import BaseModel


class LightClientBlockLiteView(BaseModel):
    inner_lite: BlockHeaderInnerLiteView
    inner_rest_hash: CryptoHash
    prev_block_hash: CryptoHash



from near_jsonrpc_models.block_id import BlockId
from near_jsonrpc_models.finality import Finality
from near_jsonrpc_models.sync_checkpoint import SyncCheckpoint
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class RpcBlockRequestBlockId(BaseModel):
    block_id: BlockId

class RpcBlockRequestFinality(BaseModel):
    finality: Finality

class RpcBlockRequestSyncCheckpoint(BaseModel):
    sync_checkpoint: SyncCheckpoint

class RpcBlockRequest(RootModel[Union[RpcBlockRequestBlockId, RpcBlockRequestFinality, RpcBlockRequestSyncCheckpoint]]):
    pass




from near_jsonrpc_models.execution_outcome_with_id_view import ExecutionOutcomeWithIdView
from near_jsonrpc_models.light_client_block_lite_view import LightClientBlockLiteView
from near_jsonrpc_models.merkle_path_item import MerklePathItem
from pydantic import BaseModel
from typing import List


class RpcLightClientExecutionProofResponse(BaseModel):
    block_header_lite: LightClientBlockLiteView
    block_proof: List[MerklePathItem]
    outcome_proof: ExecutionOutcomeWithIdView
    outcome_root_proof: List[MerklePathItem]



from near_jsonrpc_models.rpc_send_transaction_request import RpcSendTransactionRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForBroadcastTxAsync(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['broadcast_tx_async']
    params: RpcSendTransactionRequest



"""Contains main info about the chunk."""

from near_jsonrpc_models.bandwidth_requests import BandwidthRequests
from near_jsonrpc_models.congestion_info_view import CongestionInfoView
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.near_gas import NearGas
from near_jsonrpc_models.near_token import NearToken
from near_jsonrpc_models.shard_id import ShardId
from near_jsonrpc_models.signature import Signature
from near_jsonrpc_models.validator_stake_view import ValidatorStakeView
from pydantic import BaseModel
from pydantic import Field
from pydantic import conint
from typing import List


class ChunkHeaderView(BaseModel):
    balance_burnt: NearToken
    bandwidth_requests: BandwidthRequests | None = None
    chunk_hash: CryptoHash
    congestion_info: CongestionInfoView | None = None
    encoded_length: conint(ge=0, le=18446744073709551615)
    encoded_merkle_root: CryptoHash
    gas_limit: NearGas
    gas_used: NearGas
    height_created: conint(ge=0, le=18446744073709551615)
    height_included: conint(ge=0, le=18446744073709551615)
    outcome_root: CryptoHash
    outgoing_receipts_root: CryptoHash
    prev_block_hash: CryptoHash
    prev_state_root: CryptoHash
    # TODO(2271): deprecated.
    rent_paid: NearToken = Field(default_factory=lambda: NearToken('0'))
    shard_id: ShardId
    signature: Signature
    tx_root: CryptoHash
    validator_proposals: List[ValidatorStakeView]
    # TODO(2271): deprecated.
    validator_reward: NearToken = Field(default_factory=lambda: NearToken('0'))



from near_jsonrpc_models.rpc_transaction_status_request import RpcTransactionStatusRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForTx(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['tx']
    params: RpcTransactionStatusRequest



from pydantic import BaseModel
from pydantic import RootModel
from typing import Any
from typing import Dict
from typing import Literal
from typing import Union


class RpcBlockErrorUnknownBlock(BaseModel):
    info: Dict[str, Any]
    name: Literal['UNKNOWN_BLOCK']

class RpcBlockErrorNotSyncedYet(BaseModel):
    name: Literal['NOT_SYNCED_YET']

class RpcBlockErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcBlockErrorInternalError(BaseModel):
    info: RpcBlockErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcBlockError(RootModel[Union[RpcBlockErrorUnknownBlock, RpcBlockErrorNotSyncedYet, RpcBlockErrorInternalError]]):
    pass




from near_jsonrpc_models.error_wrapper_for_rpc_view_gas_key_list_error import ErrorWrapperForRpcViewGasKeyListError
from near_jsonrpc_models.rpc_view_gas_key_list_response import RpcViewGasKeyListResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcViewGasKeyListResponseAndRpcViewGasKeyListErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcViewGasKeyListResponse

class JsonRpcResponseForRpcViewGasKeyListResponseAndRpcViewGasKeyListErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcViewGasKeyListError

class JsonRpcResponseForRpcViewGasKeyListResponseAndRpcViewGasKeyListError(RootModel[Union[JsonRpcResponseForRpcViewGasKeyListResponseAndRpcViewGasKeyListErrorResult, JsonRpcResponseForRpcViewGasKeyListResponseAndRpcViewGasKeyListErrorError]]):
    pass




from near_jsonrpc_models.rpc_view_gas_key_list_request import RpcViewGasKeyListRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalViewGasKeyList(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_view_gas_key_list']
    params: RpcViewGasKeyListRequest
